﻿//===================================================================================================================
//  File:       Book.cs
//  Author:     Dan Becker, Dave Langer 
//  Course:     CSS 590 A - Mobile Computing, Spring 2012
//===================================================================================================================
using System;
using System.Json;
using System.Text;
using System.Windows;
using System.Windows.Media.Imaging;

namespace ArchitectsUnited.Phix.Model
{
    /// <summary>
    /// Abstraction of a book (i.e., Volume) returned from the Google Books API
    /// </summary>
    public class Book
    {
        #region Constants

        // String constants for grabbing data out of Book JSON
        // see Google Books API resource representation reference https://developers.google.com/books/docs/v1/reference/volumes#resource
        private const string bookInfo = "volumeInfo";
        private const string bookTitle = "title";
        private const string bookSubtitle = "subtitle";
        private const string publishedDate = "publishedDate";
        private const string description = "description";
        private const string bookAuthors = "authors";
        private const string averageRating = "averageRating";
        private const string ratingsCount = "ratingsCount";
        private const string imageLinks = "imageLinks";
        private const string thumbnailUri = "thumbnail";
        private const string industryIdentifiers = "industryIdentifiers";
        private const string isbnType = "type";
        private const string isbn = "identifier";
        private const string isbn13 = "ISBN_13";
        private const string identifierPrefix = ":";
        private const string googleBooksInfoUri = "infoLink";
        private Uri googleBooksDefaultUri = new Uri("http://books.google.com/", UriKind.Absolute);
        //private const string coverImageUri = "medium";

        #endregion


        #region Properties

        /// <summary>
        /// Title of the book 
        /// </summary>
        public string Title { get; private set; }

        /// <summary>
        /// Subtitle of the book 
        /// </summary>
        public string Subtitle { get; private set; }

        /// <summary>
        /// Date the books was published
        /// </summary>
        public string PublishedDate { get; private set; }

        /// <summary>
        /// A brief description of the book
        /// </summary>
        public string Description { get; private set; }

        /// <summary>
        /// The authors of the book as a comma-separated list
        /// </summary>
        public string Authors { get; private set; }

        /// <summary>
        /// The ISBN for the book, preferably ISBN 13
        /// </summary>
        public string ISBN { get; private set; }

        /// <summary>
        /// The average rating of the book by Google users
        /// </summary>
        public string AverageRating { get; private set; }

        /// <summary>
        /// The number of Google users that have rated the book
        /// </summary>
        public string RatingsCount { get; private set; }

        /// <summary>
        /// The thumbnail image of the book's cover (~128px wide)
        /// </summary>
        public BitmapImage Thumbnail { get; private set; }

        /// <summary>
        /// Uri to view information about this volume on the Google Books site.
        /// </summary>
        public Uri GoogleBooksUri { get; private set; }

        /// <summary>
        /// The bigger image of the book's cover (~300px wide)
        /// </summary>
        //public BitmapImage CoverImage { get; private set; }



        #endregion


        #region Constructor

        /// <summary>
        /// Instantiate a Book model object from a JSON string.
        /// </summary>
        /// <param name="bookJson">A JSON string containing the data for a single book (i.e., Volume) returned from the
        /// Google Books API.</param>
        public Book(JsonObject bookJson)
        {
            // Initialize properties
            Title = string.Empty;
            Subtitle = string.Empty;
            PublishedDate = string.Empty;
            Description = string.Empty;
            Authors = string.Empty;
            ISBN = string.Empty;
            AverageRating = string.Empty;
            RatingsCount = string.Empty;
            Thumbnail = null;
            GoogleBooksUri = googleBooksDefaultUri;
            //CoverImage = null;

            // JSON from Google is not consistent, be defensive
            if (bookJson.ContainsKey(bookInfo))
            {
                JsonObject bookInfoObj = bookJson[bookInfo] as JsonObject;

                // Get the book's title
                if (bookInfoObj.ContainsKey(bookTitle))
                {
                    Title = bookInfoObj[bookTitle];
                }

                // Get the book's subtitle
                if (bookInfoObj.ContainsKey(bookSubtitle))
                {
                    Subtitle = bookInfoObj[bookSubtitle];
                }

                // Get the book's published date
                if (bookInfoObj.ContainsKey(publishedDate))
                {
                    PublishedDate = bookInfoObj[publishedDate];
                }

                // Get the book's description
                if (bookInfoObj.ContainsKey(description))
                {
                    Description = bookInfoObj[description];
                }

                // Ensure that every book has a thumbnail for UI presentation
                Uri bookThumbnailUri = null;

                if (bookInfoObj.ContainsKey(imageLinks) && bookInfoObj[imageLinks].ContainsKey(thumbnailUri))
                {
                    // Google Books API returned a Thumbnail URI
                    bookThumbnailUri = new Uri((bookInfoObj[imageLinks])[thumbnailUri], UriKind.RelativeOrAbsolute);
                }
                else
                {
                    // Use generic cover
                    bookThumbnailUri = new Uri("/Assets/Images/NoCoverGrey_128px.png", UriKind.Relative);
                }

                if (bookInfoObj.ContainsKey(googleBooksInfoUri))
                {
                    GoogleBooksUri = new Uri((bookInfoObj[googleBooksInfoUri]), UriKind.Absolute);
                }

                // Apparently we need to download images on the UI thread.
                Deployment.Current.Dispatcher.BeginInvoke(() =>
                {
                    // Get the URI from the JSON
                    Thumbnail = new BitmapImage(bookThumbnailUri);
                    //CoverImage = new BitmapImage(bookCoverImageUri);
                });

                // Get the book's author(s)
                if (bookInfoObj.ContainsKey(bookAuthors))
                {
                    StringBuilder authors = new StringBuilder();
                    int counter = 0;

                    // There can be more than one author, build a comma-separated string
                    foreach (JsonPrimitive authorJson in (bookJson[bookInfo])[bookAuthors])
                    {
                        authors.Append((string)authorJson);

                        if (++counter < (bookJson[bookInfo])[bookAuthors].Count)
                        {
                            authors.Append(", ");
                        }
                    }

                    Authors = authors.ToString();
                }

                // Get the ISBN for the book
                if (bookInfoObj.ContainsKey(industryIdentifiers))
                {
                    foreach (JsonObject identifier in bookInfoObj[industryIdentifiers])
                    {
                        ISBN = identifier[isbn];

                        if (identifier[isbnType] == isbn13)
                        {
                            break;
                        }
                    }

                    // If "other" identifier used, get rid of prefix (e.g., "OCLC")
                    if (ISBN.Contains(identifierPrefix))
                    {
                        ISBN = ISBN.Substring(ISBN.IndexOf(identifierPrefix) + 1);
                    }
                }

                // Get the book's average rating
                if (bookInfoObj.ContainsKey(averageRating))
                {
                    AverageRating = bookInfoObj[averageRating].ToString();
                }

                // Get the book's ratings count
                if (bookInfoObj.ContainsKey(ratingsCount))
                {
                    RatingsCount = bookInfoObj[ratingsCount].ToString();
                }
            }
        }

        #endregion
    }
}
