﻿//===================================================================================================================
//  File:       WorldCatLibrary.cs
//  Authors:    Dan Becker, Dave Langer 
//  Course:     CSS 590 A - Mobile Computing, Spring 2012
//===================================================================================================================
using System;

namespace ArchitectsUnited.Phix.Model
{
    /// <summary>
    /// A simple model class that represents a single library result returned from the WorldCat servlet. Object data
    /// is parsed from a HTML fragement returned from the servlet.
    /// </summary>
    public class WorldCatLibrary
    {
        #region Constants

        // Search strings for name parsing
        private const string nameBeginParseString1 = "<a";
        private const string nameBeginParseString2 = ">";
        private const string nameEndParseString = "</a>";

        // Search strings for distance parsing
        private const string distanceBeginParseString = "<td class=\"dist\">";
        private const string distanceEndParseString = "<div id=\"mapItLink_";

        // Search strings for location parsing
        private const string locationBeginParseString = "<p class=\"geoloc\">";
        private const string locationEndParseString = "</p>";

        // Search strings for MapCode parsing
        private const string mapCodeBeginParseString = "getMapItURL('";
        private const string mapCodeEndParseString = "')";

        // Search string for Library URI parsing
        private const string libraryUriBeginParseString = "<a href=\"/wcpa/oclc/";
        private const string libraryUriEndParseString = "\"";
        private const string libraryUriAdjustString = "/wcpa/oclc/";    // We substract this off of the begin position

        // Distance parsing
        private const string distanceParseString = " miles";

        // Utility constants
        private const string worldCatAbsoluteUri = "http://www.worldcat.org";
        private const string defaultLibraryUri = "https://www.google.com/search?q={0}";

        #endregion


        #region Properties

        /// <summary>
        /// The name of the library as parsed from the WorldCat HTML fragment
        /// </summary>
        public string Name { get; private set; }

        /// <summary>
        /// The distance of the library from the user's current zip code as parsed from the WorldCat HTML fragment
        /// </summary>
        public string Distance { get; private set; }

        /// <summary>
        /// The distance of the library from the user's current zip code as parsed from the WorldCat HTML fragment
        /// </summary>
        public int DistanceAsInt 
        {
            get
            {
                if (string.IsNullOrEmpty(Distance))
                {
                    // When there is no distance provided by WorldCat, assume max int
                    return int.MaxValue;
                }
                else
                {
                    return int.Parse(Distance.Replace(distanceParseString, string.Empty));
                }
            }
        }

        /// <summary>
        /// The location of the library as parsed from the WorldCat HTML fragment
        /// </summary>
        public string Location { get; private set; }

        /// <summary>
        /// The maping code used by WorldCat to designate the Google Maps information for a specific library
        /// </summary>
        public string MapCode { get; private set; }

        /// <summary>
        /// The URI for the Library as provided by WorldCat. If no URI is provided, will default to Google search string for
        /// Library
        /// </summary>
        public string LibraryUri { get; private set; }

        /// <summary>
        /// Uri to the library's logo - will be customized for our favorite local libraries; generic otherwise (~100px wide)
        /// </summary>
        public Uri LogoUri { get; private set; }

        #endregion


        #region Constructor

        /// <summary>
        /// Instantiate an object representation of a WorldCat library.
        /// </summary>
        /// <param name="htmlFragment">Valid HTML fragment from the WorldCat servlet that returns Library data for a particular book.</param>
        public WorldCatLibrary(string htmlFragment)
        {
            if (String.IsNullOrEmpty(htmlFragment))
            {
                throw new ArgumentException("htmlFragement cannot be null/empty!");
            }

            // Parse WorldCat HTML fragment
            ParseLibraryName(htmlFragment);
            ParseLibraryDistance(htmlFragment);
            ParseLibraryLocation(htmlFragment);
            ParseMapCode(htmlFragment);
            ParseLibraryUri(htmlFragment);
            SetLogo();
        }

        #endregion


        #region Common Helper Methods

        /// <summary>
        /// Set custom logo image for our favorite local libraries; generic logo for others
        /// 
        /// NB: if phix was going to support many custom libraries from many geographies it would make sense 
        /// to retrieve these icons from the web and let phix cache locally only those icons it actually uses.
        /// </summary>
        private void SetLogo()
        {
            //Uri mLogoUri = null;
            if (Name.Contains("Seattle Public"))
                LogoUri = new Uri("/Assets/Images/seattle-public-library_photo_100px.png", UriKind.Relative);
            //else if (Name.Contains("King County"))
            //    // King County library does not have a direct link in worldcat, so lets take away their custom icon
            //    // Don't want to encourage people to click on KCLS since it'll just get them a google search 
            //    LogoUri = new Uri("/Assets/Images/King_County_library_logo_100px.png", UriKind.Relative);
            else if (Name.Contains("University of Washington"))
                LogoUri = new Uri("/Assets/Images/uw_suzallo_library_reading_room_100px.png", UriKind.Relative);
            else if (Name.Contains("Seattle University"))
                LogoUri = new Uri("/Assets/Images/SeattleUniversitySeal_100px.png", UriKind.Relative);
            else if (Name.Contains("North Seattle Community"))
                LogoUri = new Uri("/Assets/Images/North_Seattle_Commnunity_College_Logo_100px.png", UriKind.Relative);
            else if (Name.Contains("Kitsap Regional"))
                LogoUri = new Uri("/Assets/Images/kitsap_regional_library_logo_100px.png", UriKind.Relative);
            else if (Name.Contains("Bellevue Community"))
                LogoUri = new Uri("/Assets/Images/bellevue_college_100px.png", UriKind.Relative);
            else if (Name.Contains("Seattle Central"))
                LogoUri = new Uri("/Assets/Images/seattle_central_community_college.png", UriKind.Relative);
            else if (Name.Contains("Seattle Pacific"))
                LogoUri = new Uri("/Assets/Images/seattle_pacific_u_logo_100px.png", UriKind.Relative);
            else if (Name.Contains("Shoreline Community"))
                LogoUri = new Uri("/Assets/Images/shoreline_community_college_100px.png", UriKind.Relative);
            else
                LogoUri = new Uri("/Assets/Images/library_pixelfactory_100px.png", UriKind.Relative);
        }


        /// <summary>
        /// Helper method to parse out the name of the library from the WorldCat HTML fragment.
        /// </summary>
        private void ParseLibraryName(string htmlFragment)
        {
            int start = htmlFragment.IndexOf(nameBeginParseString1);

            if (start == -1)
            {
                throw new ArgumentException(String.Format("WorldCat HTML Fragment invalid! HTML:\n{0}", htmlFragment));
            }

            start = htmlFragment.IndexOf(nameBeginParseString2, start);

            int end = htmlFragment.IndexOf(nameEndParseString, start);
            
            // Name of library is located beween the <a></a> tags in HTML fragment
            Name = htmlFragment.Substring(start + nameBeginParseString2.Length,
                                          end - (start + nameBeginParseString2.Length)).Trim();
        }

        /// <summary>
        /// Helper method to parse out the distance of the library from the WorldCat HTML fragment.
        /// </summary>
        private void ParseLibraryDistance(string htmlFragment)
        {
            int start = htmlFragment.IndexOf(distanceBeginParseString);

            if (start == -1)
            {
                throw new ArgumentException(String.Format("WorldCat HTML Fragment invalid! HTML:\n{0}", htmlFragment));
            }

            int end = htmlFragment.IndexOf(distanceEndParseString, start);

            // Distance of library is located immediately after <td class="dist"> tag
            Distance = htmlFragment.Substring(start + distanceBeginParseString.Length,
                                              end - (start + distanceBeginParseString.Length)).Trim();
        }

        /// <summary>
        /// Helper method to parse out the location of the library from the WorldCat HTML fragment.
        /// </summary>
        private void ParseLibraryLocation(string htmlFragment)
        {
            int start = htmlFragment.IndexOf(locationBeginParseString);

            if (start == -1)
            {
                throw new ArgumentException(String.Format("WorldCat HTML Fragment invalid! HTML:\n{0}", htmlFragment));
            }

            int end = htmlFragment.IndexOf(locationEndParseString, start);

            // Distance of library is located immediately after <p class="geoloc"> tag
            Location = htmlFragment.Substring(start + locationBeginParseString.Length,
                                              end - (start + locationBeginParseString.Length)).Trim();
        }

        /// <summary>
        /// Helper method to parse out the map code of the library from the WorldCat HTML fragment.
        /// </summary>
        private void ParseMapCode(string htmlFragment)
        {
            int start = htmlFragment.IndexOf(mapCodeBeginParseString);

            if (start == -1)
            {
                throw new ArgumentException(String.Format("WorldCat HTML Fragment invalid! HTML:\n{0}", htmlFragment));
            }

            int end = htmlFragment.IndexOf(mapCodeEndParseString, start);

            // Distance of library is located immediately after <p class="geoloc"> tag
            MapCode = htmlFragment.Substring(start + mapCodeBeginParseString.Length,
                                             end - (start + mapCodeBeginParseString.Length)).Trim();
        }

        /// <summary>
        /// Helper method to parse out the Library's Uri from the WorldCat HTML fragement.
        /// </summary>
        private void ParseLibraryUri(string htmlFragment)
        {
            int start = htmlFragment.IndexOf(libraryUriBeginParseString);

            if (start == -1)
            {
                // Sometime WorldCat doesn't provide a URI for the library, use a Google search string instead
                LibraryUri = string.Format(defaultLibraryUri, Uri.EscapeDataString(Name));
            }
            else
            {
                int end = htmlFragment.IndexOf(libraryUriEndParseString, start + libraryUriAdjustString.Length);

                // Library URI is denoted by the following tag fragment "<a href="/wcpa/oclc/". Note that we have to
                // account for both "http://www.worldcat.org" and "/wcpa/oclc" in the resulting URI string
                LibraryUri = worldCatAbsoluteUri +
                             htmlFragment.Substring(start + libraryUriBeginParseString.Length - libraryUriAdjustString.Length,
                                                    end - (start + libraryUriBeginParseString.Length - libraryUriAdjustString.Length)).Trim();
            }
        }

        #endregion
    }
}
