﻿//===================================================================================================================
//  File:       WorldCatSeller.cs
//  Authors:    Dan Becker, Dave Langer 
//  Course:     CSS 590 A - Mobile Computing, Spring 2012
//===================================================================================================================
using System;

namespace ArchitectsUnited.Phix.Model
{
    /// <summary>
    /// A simple model class that represents a single seller (e.g., Amazon) result returned from the WorldCat servlet. Object data
    /// is parsed from a HTML fragement returned from the servlet.
    /// </summary>
    public class WorldCatSeller
    {
        #region Constants

        private const string sellerBeginParseString1 = "href=\"";
        private const string sellerBeginParseString2 = ">";
        private const string sellerEndParseString = "</a>";
        private const string uriEndParseString = "\"";
        private const string priceBeginParseString = "<td class=\"price\">";
        private const string tableDataEndParseString = "</td>";

        #endregion


        #region Properties

        /// <summary>
        /// The the name of the WorldCatSeller
        /// </summary>
        public string Name { get; private set; }

        /// <summary>
        /// The price the WorldCatSeller is selling the book for
        /// </summary>
        public string Price { get; private set; }

        /// <summary>
        /// The URI string to the seller's site for the book.
        /// 
        /// NOTE - We leave WorldCat affiliate info in the URL for their benefit.
        /// </summary>
        public string Uri { get; private set; }

        /// <summary>
        /// Uri to the store's logo - will be customized for top sellers; generic otherwise (~100px wide)
        /// </summary>
        public Uri LogoUri { get; private set; }


        #endregion


        #region Constructor

        /// <summary>
        /// Instantiate an object representation of a WorldCat Seller.
        /// </summary>
        /// <param name="htmlFragment">Valid HTML fragment from the WorldCat servlet that returns seller data for a particular book.</param>
        public WorldCatSeller(string htmlFragment)
        {
            ParseSellerName(htmlFragment);
            ParsePrice(htmlFragment);
            ParseUri(htmlFragment);
            SetLogo(); 

        }
        #endregion


        #region Methods

        /// <summary>
        /// Set custom logo image for top sellers; generic logo for others
        /// </summary>
        private void SetLogo()
        {
            //Uri mLogoUri = null;
            if (Name.Contains("Amazon"))
                LogoUri = new Uri("/Assets/Images/amazon_logo_black_button_trans_100px.png", UriKind.Relative);
            else if (Name.Contains ("Barnes"))
                LogoUri = new Uri("/Assets/Images/b&n_logo_button_trans_100px.png", UriKind.Relative);
            else if (Name.Contains ("Better World Books"))
                LogoUri = new Uri("/Assets/Images/BWB_Logo_trans_100px.png", UriKind.Relative);
            else
                LogoUri = new Uri("/Assets/Images/bookcase_David_Vignoni_Nuvola_100px.png", UriKind.Relative);
        }
  
        /// <summary>
        /// Helper method to parse out the name of the library from the WorldCat HTML fragment.
        /// </summary>
        private void ParseSellerName(string htmlFragment)
        {
            int start = htmlFragment.IndexOf(sellerBeginParseString1);

            if (start == -1)
            {
                throw new ArgumentException(String.Format("WorldCat HTML Fragment invalid! HTML:\n{0}", htmlFragment));
            }

            start = htmlFragment.IndexOf(sellerBeginParseString2, start);

            int end = htmlFragment.IndexOf(sellerEndParseString, start);

            // Name of library is located beween the <a></a> tags in HTML fragment
            Name = (htmlFragment.Substring(start + sellerBeginParseString2.Length,
                                           end - (start + sellerBeginParseString2.Length)));

            // Just in case, replace HTML escaping for & and the trade mark symbol
            Name = Name.Replace("&amp;", "&");
            Name = Name.Replace("&trade;", "");
        }

        /// <summary>
        /// Helper method to parse out the price of the book from the WorldCat HTML fragment.
        /// </summary>
        private void ParsePrice(string htmlFragment)
        {
            int start = htmlFragment.IndexOf(priceBeginParseString);

            if (start == -1)
            {
                throw new ArgumentException(String.Format("WorldCat HTML Fragment invalid! HTML:\n{0}", htmlFragment));
            }

            int end = htmlFragment.IndexOf(tableDataEndParseString, start);

            // Price of book is located beween the <td></td> tags in HTML fragment
            Price = htmlFragment.Substring(start + priceBeginParseString.Length,
                                           end - (start + priceBeginParseString.Length));
        }

        /// <summary>
        /// Helper method to parse out the price of the book from the WorldCat HTML fragment.
        /// </summary>
        private void ParseUri(string htmlFragment)
        {
            int start = htmlFragment.IndexOf(sellerBeginParseString1);

            if (start == -1)
            {
                throw new ArgumentException(String.Format("WorldCat HTML Fragment invalid! HTML:\n{0}", htmlFragment));
            }

            int end = htmlFragment.IndexOf(uriEndParseString, start + sellerBeginParseString1.Length + 1);

            // Price of book is located beween the <td></td> tags in HTML fragment
            Uri = htmlFragment.Substring(start + sellerBeginParseString1.Length,
                                         end - (start + sellerBeginParseString1.Length));
        }

        #endregion
    }
}
