﻿//===================================================================================================================
//  File:       ViewModel.cs
//  Authors:    Dan Becker, Dave Langer 
//  Course:     CSS 590 A - Mobile Computing, Spring 2012
//===================================================================================================================
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Device.Location;
using System.IO;
using System.Json;
using System.Linq;
using System.Net;
using System.Windows;
using System.Xml.Linq;
using ArchitectsUnited.Phix.Model;

namespace ArchitectsUnited.Phix.ViewModel
{
    /// <summary>
    /// A ViewModel class, per the Model View ViewModel (MVVM) design pattern (a WP7 best practice). The ViewModel
    /// class provides the "business logic" of the Phix application, allowing the XAML-based pages to be solely focused
    /// on presentation layer concerns.
    /// </summary>
    public class PhixViewModel
    {
        #region Constants

        // Google Books API URIs and API Key string for conducting book searches
        private const string googleBookSearchUri = "https://www.googleapis.com/books/v1/volumes?q={0}&langRestrict=en&printType=books&maxResults=40&startIndex={1}&key={2}";
        
        //
        //  TODO - Add your Google Books API Developer Key here
        //
        private const string googleApiKey = "";

        // Search strings for Google Books JSON parsing
        private const string returnedBooks = "items";

        // Bing Maps API URIs and Deeloper Key strings for finding zip code based on location
        private const string bingMapsZipCodeUri = "http://dev.virtualearth.net/REST/v1/Locations/{0},{1}?includeEntityTypes=Address&o=xml&key={2}";
        
        //
        //  TODO - Add you Bing Maps Developer Key Here
        //
        private const string bingMapsKey = "";

        // Bing Maps XML Namespec
        private const string bingMapsNS = "{http://schemas.microsoft.com/search/local/ws/rest/v1}";

        //
        //  TODO - Add your WorldCat Basic API Developer Key to URI below
        //
        // The WorldCat Basic API URI
        private const string worldCatBasicUri = @"http://www.worldcat.org/webservices/catalog/search/opensearch?q={0}&start=1&count=100&wskey=<Your Developer Key Here>";

        // URI for the WorldCat servlet that returns library-specific information for a book.
        private const string worldCatLibraryServletUri = @"http://www.worldcat.org/wcpa/servlet/org.oclc.lac.ui.ajax.ServiceServlet?wcoclcnum={0}&loc={1}&serviceCommand=holdingsdata";

        // URI for the WorldCat servlet that returns seller information for a book.
        private const string worldCatSellerServletUri = @"http://www.worldcat.org/wcpa/servlet/org.oclc.lac.ui.buying.AjaxBuyingLinksServlet?serviceCommand=getBuyingLinks&oclcno={0}";

        // URI for the WorldCat servlet that returns Library location data.
        private const string worldCatLocationServletUri = @"http://www.worldcat.org/wcpa/servlet/org.oclc.lac.ui.ajax.ServiceServlet?serviceCommand=getMapItURL&symbol={0}";

        // Search strings to use in Bing Maps XDocument.Descendants() method call
        private const string zipCodeSearch = "{0}PostalCode";

        // Search strings for WorldCat parsing
        private const string entrySearch = "{http://www.w3.org/2005/Atom}entry";
        private const string entryFilter = "LCCN";
        private const string tableRowBeginParseString = "<tr>";
        private const string tableRowEndParseString = "</tr>";
        private const string libraryParseString = "<p class=\"lib\">";
        private const string locationBeginParseString = "@";
        private const string locationEndParseString = "&";
        private const string sellerParseString = "<td class=\"seller\">";

        // User-friendly exception strings
        private const string googleSearchErrorString = "Error encountered with the Google Books Search API. Please verify Internet connectivity and try again.";
        private const string bingMapsErrorString = "Error connecting to the Bing Maps API. Please verify Internet connectivity and try again.";
        private const string worldCatSearchErrorString = "Error connecting to the WorldCat search service. Please verify Internet connectivity and try again.";
        private const string libraryMapErrorString = "There was an error attempting to map the selected Library. Please try again.";
        private const string notFoundErrorString = "No U.S. libraries or sellers were found for this book. Please search again.";

        // Formatted full title of the book
        private const string formattedFullTitle = "{0} : {1}";

        #endregion


        #region Events

        /// <summary>
        /// Public event fired by the ViewModel when asynchronous seaching via Google Books API has been completed.
        /// </summary>
        public event EventHandler SearchCompleted;

        /// <summary>
        /// Public event fired by the ViewModel when asynchronous loading of WorldCat libraries and sellers has been completed.
        /// </summary>
        public event EventHandler LibrariesLoaded;

        /// <summary>
        /// Public event fired by the ViewModel when the Bing Maps Location API asynchronously returns with location
        /// data.
        /// </summary>
        public event EventHandler<LibraryMappedEvent> LibraryMapped;

        /// <summary>
        /// Public event fired by the ViewModel when an error is encountered (e.g., exception thrown during HTTP
        /// communication).
        /// </summary>
        public event EventHandler<ErrorEncounteredEvent> ErrorEncountered;

        #endregion


        #region Properties

        /// <summary>
        /// The current Title of the book to use for WorldCat search
        /// </summary>
        private string Title { get; set; }

        /// <summary>
        /// The current Title of the book, without punctuation (e.g., colons)
        /// </summary>
        private string StrippedTitle { get; set; }

        /// <summary>
        /// The current Sub-title of the book to use for WorldCat search
        /// </summary>
        private string Subtitle { get; set; }

        /// <summary>
        /// The current formatted full title of the book to use for WorldCat search
        /// </summary>
        private string FullTitle { get; set; }

        /// <summary>
        /// The Google Books API only provides ISBN numbers for books
        /// </summary>
        private string ISBN { get; set; }

        /// <summary>
        /// Leverage WP7 location services to determine where the user is so we can find the closest libraries for
        /// the books the user is interested in.
        /// </summary>
        private GeoCoordinateWatcher geoWatcher = null;
        private GeoCoordinateWatcher GeoWatcher
        {
            get
            {
                if (geoWatcher == null)
                {
                    // Due to limitations of WorldCat (i.e., distances are based on the centers of zip codes), 
                    // only need default accuracy. As a bonus we save some battery usage as well.
                    geoWatcher = new GeoCoordinateWatcher(GeoPositionAccuracy.Default);
                }

                return geoWatcher;
            }
        }

        /// <summary>
        /// The user's current location mapped to U.S. postal code
        /// </summary>
        private string ZipCode { get; set; }

        /// <summary>
        /// Query the ViewModel for the developer Bing Maps key
        /// </summary>
        public string BingMapsKey
        {
            get
            {
                return bingMapsKey;
            }
        }

        /// <summary>
        /// An ObservableCollection of Book instances used for Data Binding to the UI
        /// </summary>
        private ObservableCollection<Book> books = null;
        public ObservableCollection<Book> Books
        {
            get
            {
                if (books == null)
                {
                    books = new ObservableCollection<Book>();
                }

                return books;
            }
        }

        /// <summary>
        /// Collection of the current set of WorldCatSearchResults
        /// </summary>
        private List<WorldCatSearchResult> worldCatSearchResults = null;
        private IList<WorldCatSearchResult> WorldCatSearchResults
        {
            get
            {
                if (worldCatSearchResults == null)
                {
                    worldCatSearchResults = new List<WorldCatSearchResult>();
                }

                return worldCatSearchResults;
            }
        }

        /// <summary>
        /// An ObservableCollection of WorldCatLibrary instances used for Data Binding to the UI
        /// </summary>
        private ObservableCollection<WorldCatLibrary> worldCatLibraries = null;
        public ObservableCollection<WorldCatLibrary> WorldCatLibraries
        {
            get
            {
                if (worldCatLibraries == null)
                {
                    worldCatLibraries = new ObservableCollection<WorldCatLibrary>();
                }

                return worldCatLibraries;
            }
        }

        /// <summary>
        /// An ObservableCollection of WorldCatSeller instances used for Data Binding to the UI
        /// </summary>
        private ObservableCollection<WorldCatSeller> worldCatSellers = null;
        public ObservableCollection<WorldCatSeller> WorldCatSellers
        {
            get
            {
                if (worldCatSellers == null)
                {
                    worldCatSellers = new ObservableCollection<WorldCatSeller>();
                }

                return worldCatSellers;
            }
        }

        #endregion


        #region Seach Google Books API

        /// <summary>
        /// Asynchronously call WorldCat servlet to get back Library location data
        /// </summary>
        public void SearchForBook(string searchString)
        {
            // Step 1 - Asynchronously call Google Book API to search for the passed-in string
            WebRequest request = HttpWebRequest.Create(String.Format(googleBookSearchUri, Uri.EscapeUriString(searchString), 0, googleApiKey));

            request.BeginGetResponse(GoogleSearchResponseHandler, request);
        }

        /// <summary>
        /// Event handler for the response from the World Cat location servlet
        /// </summary>
        private void GoogleSearchResponseHandler(IAsyncResult result)
        {
            try
            {
                // Step 2 - Load up the JSON response and update ObservableCollection of books
                WebResponse response = ((HttpWebRequest)result.AsyncState).EndGetResponse(result);
                JsonObject bookListJson = JsonObject.Load(response.GetResponseStream()) as JsonObject;

                // Clear the current observable collection to refresh from Google Books API
                Deployment.Current.Dispatcher.BeginInvoke(() =>
                {
                    Books.Clear();
                });

                // Check to see if we got any results back from Google
                if (bookListJson.ContainsKey(returnedBooks))
                {
                    foreach (JsonObject bookJson in bookListJson[returnedBooks])
                    {
                        Book book = new Book(bookJson);

                        // WorldCat returned same book, add book to collection
                        Deployment.Current.Dispatcher.BeginInvoke(() =>
                        {
                            Books.Add(book);
                        });
                    }
                }

                // Step 3 - Asynchronous processing completed, raise event on UI thread
                Deployment.Current.Dispatcher.BeginInvoke(() =>
                {
                    SearchCompleted(this, EventArgs.Empty);
                });
            }
            catch (Exception ex)
            {
                // Since our web calls are very structured, there's nothing we can do in the event 
                // of an exception except provide friendly error string to the UI and have the user 
                // re-try.
                RaiseErrorEncounteredEvent(ErrorEncounteredEvent.ErrorType.GoogleSearch, googleSearchErrorString);
            }
        }

        #endregion


        #region Get Book from Current ObservableCollection by ISBN

        /// <summary>
        /// For a given ISBN, search the ObservableCollection and return the corresponding Book object
        /// </summary>
        /// <param name="ISBN">The Book ISBN to search for.</param>
        /// <returns>The corresponding Book instance if found, null if not.</returns>
        public Book GetBookByISBN(string ISBN)
        {
            var book = from aBook in Books
                       where aBook.ISBN == ISBN
                       select aBook;

            if (book.Count() == 0)
            {
                return null;
            }
            else
            {
                return book.First() as Book;
            }
        }


        #endregion


        #region Get WorldCat Libraries & Sellers for ISBN

        /// <summary>
        /// Implement the logic to get the six nearest libraries from WorldCat that contain the use-selected in their collections 
        /// and also get seller (e.g., Amazon) information. This logic consists of the following high level steps:
        ///     
        ///     1. Asynchronously get the users location via WP7 location services.
        ///     2. Asynchronously tanslate the latitude/longitude returned by location services to the zip code of where the user
        ///        is current located using the Bing Maps REST API.
        ///     3. Asynchronously call the WorldCat Basic API based on book title
        ///     4. Process all WorldCat search results
        ///     5. Asynchronously retrieve all libraries for all applicable search results (max of 30 libraries)
        ///     6. Return only the 6 closest libraries to the user
        ///     7. Asynchronously retrieve online sellers for the book
        /// </summary>
        /// <param name="ISBN">The identifier provided by the Google Books API to identify a book.</param>
        /// <param name="title">The title of the book as provided by the Google Books API.</param>
        /// <param name="subtitle">The subtitle of the book as provided by the Google Books API.</param>
        public void GetWorldCatLibrariesForBook(string ISBN, string title, string subtitle)
        {
            this.ISBN = ISBN;
            this.Title = title.ToLower();
            this.StrippedTitle = BuildStrippedTitle(this.Title);
            this.Subtitle = subtitle.ToLower();
            this.FullTitle = string.Format(formattedFullTitle, Title, Subtitle);

            // Clear the ObservableCollections as a visual cue to the user
            WorldCatLibraries.Clear();
            WorldCatSellers.Clear();

            // First #1 - get our current location from the phone
            GeoWatcher.PositionChanged += PositionChangedHandler;

            GeoWatcher.Start();
        }

        /// <summary>
        /// The event handler for WP7 location services PositionChanged events
        /// </summary>
        private void PositionChangedHandler(object sender, GeoPositionChangedEventArgs<GeoCoordinate> e)
        {
            try
            {
                // We just need to know where we're at, turn off the geo watcher
                GeoWatcher.PositionChanged -= PositionChangedHandler;
                GeoWatcher.Stop();

                // Step 2 - Asynchronously call Bing Maps REST API to get the zip code of our position
                WebRequest request = HttpWebRequest.Create(String.Format(bingMapsZipCodeUri,
                                                                            e.Position.Location.Latitude.ToString(),
                                                                            e.Position.Location.Longitude.ToString(),
                                                                            bingMapsKey));

                request.BeginGetResponse(BingMapsZipCodeResponseHandler, request);
            }
            catch (Exception ex)
            {
                // Since our web calls are very structured, there's nothing we can do in the event 
                // of an exception except provide friendly error string to the UI and have the user 
                // re-try.
                RaiseErrorEncounteredEvent(ErrorEncounteredEvent.ErrorType.BingMaps, bingMapsErrorString);
            }
        }

        /// <summary>
        /// Event handler for the response from the Bing Maps REST API for the Zip Code lookup based
        /// on phone's position
        /// </summary>
        private void BingMapsZipCodeResponseHandler(IAsyncResult result)
        {
            try
            {
                // Get the Postal Code returned by Bing Maps
                WebResponse response = ((HttpWebRequest)result.AsyncState).EndGetResponse(result);
                XDocument bingXml = XDocument.Load(response.GetResponseStream());
                var postalCodeEnumerator = bingXml.Descendants(String.Format(zipCodeSearch, bingMapsNS)).GetEnumerator();

                if (postalCodeEnumerator.MoveNext())
                {
                    ZipCode = postalCodeEnumerator.Current.Value;
                }
                else
                {
                    // It's possible for the Bing Maps REST API to not return a postal code
                    ZipCode = string.Empty;
                }

                // Step 3 - Asynchronously call the WorldCat Basic API to search on the book's Title
                WebRequest request = HttpWebRequest.Create(String.Format(worldCatBasicUri, Uri.EscapeUriString(Title)));

                request.BeginGetResponse(WorldCatBasicResponseHandler, request);
            }
            catch (Exception ex)
            {
                // Since our web calls are very structured, there's nothing we can do in the event 
                // of an exception except provide friendly error string to the UI and have the user 
                // re-try.
                RaiseErrorEncounteredEvent(ErrorEncounteredEvent.ErrorType.BingMaps, bingMapsErrorString);
            }
        }

        /// <summary>
        /// Callback method for the HttpWebRequest that calls the WorldCat Basic API.
        /// </summary>
        private void WorldCatBasicResponseHandler(IAsyncResult result)
        {
            try
            {
                // Get the XML response and parse out the OCLC
                WebResponse response = ((HttpWebRequest)result.AsyncState).EndGetResponse(result);
                XDocument worldCatXml = XDocument.Load(response.GetResponseStream());
                IEnumerable<XElement> searchResults = worldCatXml.Descendants(entrySearch);
                List<WorldCatSearchResult> secondarySearchResults = new List<WorldCatSearchResult>();
                WorldCatSearchResult searchResult = null;

                // Clear search results collection
                WorldCatSearchResults.Clear();

                // Step 4 - Process all the search resutls returned by WorldCat
                foreach (XElement element in searchResults)
                {
                    string elementXml = element.ToString();

                    // Only take the search results that have both a Library of Congress Control Number (LCCN)
                    if (elementXml.Contains(entryFilter))
                    {
                        searchResult = new WorldCatSearchResult(element);
                        string resultTitle = searchResult.Title.ToLower();
                        string strippedResultTitle = BuildStrippedTitle(resultTitle);

                        // Only take the search results where the ISBN, formatted full title, title, or stripped title match
                        if (searchResult.ISBNs.Contains(ISBN) || resultTitle.Equals(FullTitle) || resultTitle.Equals(Title) ||
                            strippedResultTitle.Equals(StrippedTitle))
                        {
                            WorldCatSearchResults.Add(new WorldCatSearchResult(element));
                        }
                    }
                    else
                    {
                        // Build list of less desirable (i.e., secondary) search results to use
                        searchResult = new WorldCatSearchResult(element);
                        string resultTitle = searchResult.Title.ToLower();
                        string strippedResultTitle = BuildStrippedTitle(resultTitle);

                        // Only take the search results where the ISBN, formatted full title, title, or stripped title match
                        if (searchResult.ISBNs.Contains(ISBN) || resultTitle.Equals(FullTitle) || resultTitle.Equals(Title) ||
                            strippedResultTitle.Equals(StrippedTitle))
                        {
                            secondarySearchResults.Add(new WorldCatSearchResult(element));
                        }
                    }
                }

                // Merge search results
                foreach (WorldCatSearchResult sceondaryResult in secondarySearchResults)
                {
                    WorldCatSearchResults.Add(sceondaryResult);
                }

                // Despite all this work, it's possible for Google Books API to return books that don't exist in WorldCat, 
                // handle this.
                if (WorldCatSearchResults.Count == 0)
                {
                    RaiseErrorEncounteredEvent(ErrorEncounteredEvent.ErrorType.BookNotFound, notFoundErrorString);

                    return;
                }

                // Step 5 - Asynchronously call the WorldCat servlet to get the six nearest libraries for the first search result
                WebRequest request = HttpWebRequest.Create(String.Format(worldCatLibraryServletUri, WorldCatSearchResults.First().OCLC, ZipCode));

                request.BeginGetResponse(WorldCatLibraryServletResponseHandler1, request);
            }
            catch (Exception ex)
            {
                // Since our web calls are very structured, there's nothing we can do in the event 
                // of an exception except provide friendly error string to the UI and have the user 
                // re-try.
                RaiseErrorEncounteredEvent(ErrorEncounteredEvent.ErrorType.WorldCatSearch, worldCatSearchErrorString);
            }
        }

        /// <summary>
        /// Callback method for the HttpWebRequest that calls the WorldCat libraries servlet for the first search result.
        /// </summary>
        private void WorldCatLibraryServletResponseHandler1(IAsyncResult result)
        {
            try
            {
                // Get HTML fragment response and parse out the libraries
                ParseAndAddLibraries(result);

                if (WorldCatSearchResults.Count > 1)
                {
                    // Step 6 - More than one search result was returned, get more libraries to ensure the user get the closest set
                    WebRequest request = HttpWebRequest.Create(String.Format(worldCatLibraryServletUri, WorldCatSearchResults[1].OCLC, ZipCode));

                    request.BeginGetResponse(WorldCatLibraryServletResponseHandler2, request);
                }
                else
                {
                    // Step 7 - Asynchronously call the WorldCat servlet to get the sellers for the book based on first search result
                    WebRequest request = HttpWebRequest.Create(String.Format(worldCatSellerServletUri, WorldCatSearchResults.First().OCLC));

                    request.BeginGetResponse(WorldCatSellerServletResponseHandler, request);
                }
            }
            catch(Exception ex)
            {
                // Since our web calls are very structured, there's nothing we can do in the event 
                // of an exception except provide friendly error string to the UI and have the user 
                // re-try.
                RaiseErrorEncounteredEvent(ErrorEncounteredEvent.ErrorType.WorldCatSearch, worldCatSearchErrorString);
            }
        }

        /// <summary>
        /// Callback method for the HttpWebRequest that calls the WorldCat libraries servlet for the second search result.
        /// </summary>
        private void WorldCatLibraryServletResponseHandler2(IAsyncResult result)
        {
            try
            {
                // Get HTML fragment response and parse out the libraries
                ParseAndAddLibraries(result);

                if (WorldCatSearchResults.Count > 2)
                {
                    // Step 6 - More than two search results were returned, get more libraries to ensure the user gets the closest set
                    WebRequest request = HttpWebRequest.Create(String.Format(worldCatLibraryServletUri, WorldCatSearchResults[2].OCLC, ZipCode));

                    request.BeginGetResponse(WorldCatLibraryServletResponseHandler3, request);
                }
                else
                {
                    // Step 6 - Asynchronously call the WorldCat servlet to get the sellers for the book based on first search result, first
                    //          sort the list of libraries and show only the six closest
                    SortAndUpdateLibraries();

                    // Step 7 - Asynchronously call the WorldCat servlet to get the sellers for the book based on first search result
                    WebRequest request = HttpWebRequest.Create(String.Format(worldCatSellerServletUri, WorldCatSearchResults.First().OCLC));

                    request.BeginGetResponse(WorldCatSellerServletResponseHandler, request);
                }
            }
            catch (Exception ex)
            {
                // Since our web calls are very structured, there's nothing we can do in the event 
                // of an exception except provide friendly error string to the UI and have the user 
                // re-try.
                RaiseErrorEncounteredEvent(ErrorEncounteredEvent.ErrorType.WorldCatSearch, worldCatSearchErrorString);
            }
        }

        /// <summary>
        /// Callback method for the HttpWebRequest that calls the WorldCat libraries servlet for the third search result.
        /// </summary>
        private void WorldCatLibraryServletResponseHandler3(IAsyncResult result)
        {
            try
            {
                // Get HTML fragment response and parse out the libraries
                ParseAndAddLibraries(result);

                if (WorldCatSearchResults.Count > 3)
                {
                    // Step 6 - More than two search results were returned, get more libraries to ensure the user gets the closest set
                    WebRequest request = HttpWebRequest.Create(String.Format(worldCatLibraryServletUri, WorldCatSearchResults[3].OCLC, ZipCode));

                    request.BeginGetResponse(WorldCatLibraryServletResponseHandler4, request);
                }
                else
                {
                    // Step 6 - Asynchronously call the WorldCat servlet to get the sellers for the book based on first search result, first
                    //          sort the list of libraries and show only the six closest
                    SortAndUpdateLibraries();

                    // Step 7 - Asynchronously call the WorldCat servlet to get the sellers for the book based on first search result
                    WebRequest request = HttpWebRequest.Create(String.Format(worldCatSellerServletUri, WorldCatSearchResults.First().OCLC));

                    request.BeginGetResponse(WorldCatSellerServletResponseHandler, request);
                }
            }
            catch (Exception ex)
            {
                // Since our web calls are very structured, there's nothing we can do in the event 
                // of an exception except provide friendly error string to the UI and have the user 
                // re-try.
                RaiseErrorEncounteredEvent(ErrorEncounteredEvent.ErrorType.WorldCatSearch, worldCatSearchErrorString);
            }
        }

        /// <summary>
        /// Callback method for the HttpWebRequest that calls the WorldCat libraries servlet for the fourth search result.
        /// </summary>
        private void WorldCatLibraryServletResponseHandler4(IAsyncResult result)
        {
            try
            {
                // Get HTML fragment response and parse out the libraries
                ParseAndAddLibraries(result);

                if (WorldCatSearchResults.Count > 4)
                {
                    // Step 6 - More than two search results were returned, get more libraries to ensure the user gets the closest set
                    WebRequest request = HttpWebRequest.Create(String.Format(worldCatLibraryServletUri, WorldCatSearchResults[4].OCLC, ZipCode));

                    request.BeginGetResponse(WorldCatLibraryServletResponseHandler5, request);
                }
                else
                {
                    // Step 6 - Asynchronously call the WorldCat servlet to get the sellers for the book based on first search result, first
                    //          sort the list of libraries and show only the six closest
                    SortAndUpdateLibraries();

                    // Step 7 - Asynchronously call the WorldCat servlet to get the sellers for the book based on first search result
                    WebRequest request = HttpWebRequest.Create(String.Format(worldCatSellerServletUri, WorldCatSearchResults.First().OCLC));

                    request.BeginGetResponse(WorldCatSellerServletResponseHandler, request);
                }
            }
            catch (Exception ex)
            {
                // Since our web calls are very structured, there's nothing we can do in the event 
                // of an exception except provide friendly error string to the UI and have the user 
                // re-try.
                RaiseErrorEncounteredEvent(ErrorEncounteredEvent.ErrorType.WorldCatSearch, worldCatSearchErrorString);
            }
        }

        /// <summary>
        /// Callback method for the HttpWebRequest that calls the WorldCat libraries servlet for the fifth (and final) search result.
        /// </summary>
        private void WorldCatLibraryServletResponseHandler5(IAsyncResult result)
        {
            try
            {
                // Get HTML fragment response and parse out the libraries
                ParseAndAddLibraries(result);

                // Step 6 - Asynchronously call the WorldCat servlet to get the sellers for the book based on first search result, first
                //          sort the list of libraries and show only the six closest
                SortAndUpdateLibraries();

                // Step 7 - Asynchronously call the WorldCat servlet to get the sellers for the book based on first search result
                WebRequest request = HttpWebRequest.Create(String.Format(worldCatSellerServletUri, WorldCatSearchResults.First().OCLC));

                request.BeginGetResponse(WorldCatSellerServletResponseHandler, request);
            }
            catch (Exception ex)
            {
                // Since our web calls are very structured, there's nothing we can do in the event 
                // of an exception except provide friendly error string to the UI and have the user 
                // re-try.
                RaiseErrorEncounteredEvent(ErrorEncounteredEvent.ErrorType.WorldCatSearch, worldCatSearchErrorString);
            }
        }

        /// <summary>
        /// Callback method for the HttpWebRequest that calls the WorldCat sellers servlet.
        /// </summary>
        private void WorldCatSellerServletResponseHandler(IAsyncResult result)
        {
            try
            {
                // Get HTML fragment response and parse out the libraries
                WebResponse response = ((HttpWebRequest)result.AsyncState).EndGetResponse(result);
                StreamReader reader = new StreamReader(response.GetResponseStream());
                string htmlFragment = reader.ReadToEnd();
                int start = htmlFragment.IndexOf(tableRowBeginParseString);
                int end = 0;

                while (start != -1)
                {
                    end = htmlFragment.IndexOf(tableRowEndParseString, start);

                    // Seller data is contained between <tr></tr> tags in the HTML fragment
                    string temp = htmlFragment.Substring(start, end - (start + libraryParseString.Length));

                    if (temp.Contains(sellerParseString))
                    {
                        WorldCatSeller seller = new WorldCatSeller(temp);

                        // If the table row contains a <p class="seller"> tag then we've got a seller
                        // Use the UI thread to add a new seller to the ObservableCollection
                        Deployment.Current.Dispatcher.BeginInvoke(() =>
                        {
                            WorldCatSellers.Add(seller);
                        });
                    }

                    start = htmlFragment.IndexOf(tableRowBeginParseString, end);
                }

                // Asynchronous processing completed, raise event on UI thread
                Deployment.Current.Dispatcher.BeginInvoke(() =>
                {
                    LibrariesLoaded(this, EventArgs.Empty);
                });
            }
            catch (Exception ex)
            {
                // Since our web calls are very structured, there's nothing we can do in the event 
                // of an exception except provide friendly error string to the UI and have the user 
                // re-try.
                RaiseErrorEncounteredEvent(ErrorEncounteredEvent.ErrorType.WorldCatSearch, worldCatSearchErrorString);
            }
        }

        #endregion


        #region Map a WorldCat Library to Bing Maps Control

        /// <summary>
        /// Asynchronously call WorldCat servlet to get back Library location data
        /// </summary>
        public void MapLibrary(string libraryMapCode)
        {
            try
            {
                // Step 1 - Asynchronously call WorldCat Location servlet for the user-selected library
                WebRequest request = HttpWebRequest.Create(String.Format(worldCatLocationServletUri, libraryMapCode));

                request.BeginGetResponse(WorldCatLocationResponseHandler, request);
            }
            catch (Exception ex)
            {
                // Since our web calls are very structured, there's nothing we can do in the event 
                // of an exception except provide friendly error string to the UI and have the user 
                // re-try.
                RaiseErrorEncounteredEvent(ErrorEncounteredEvent.ErrorType.LibraryMap, libraryMapErrorString);
            }
        }

        /// <summary>
        /// Event handler for the response from the World Cat location servlet
        /// </summary>
        private void WorldCatLocationResponseHandler(IAsyncResult result)
        {
            try
            {
                // Get the location data for the library returned by Bing Maps
                WebResponse response = ((HttpWebRequest)result.AsyncState).EndGetResponse(result);
                StreamReader reader = new StreamReader(response.GetResponseStream());
                string locationData = reader.ReadToEnd();
                int start = locationData.IndexOf(locationBeginParseString);
                string latitude = string.Empty;
                string longitude = string.Empty;

                if (start != -1)
                {
                    int end = locationData.IndexOf(locationEndParseString, start);
                    string[] latLong = locationData.Substring(start + locationBeginParseString.Length,
                                                              end - (start + locationBeginParseString.Length)).Split(',');

                    latitude = latLong[0];
                    longitude = latLong[1];
                }
                    
                // Step 2 - Asynchronous processing completed, raise event on UI thread
                Deployment.Current.Dispatcher.BeginInvoke(() =>
                {
                    LibraryMapped(this, new LibraryMappedEvent(latitude, longitude));
                });
            }
            catch (Exception ex)
            {
                // Since our web calls are very structured, there's nothing we can do in the event 
                // of an exception except provide friendly error string to the UI and have the user 
                // re-try.
                RaiseErrorEncounteredEvent(ErrorEncounteredEvent.ErrorType.LibraryMap, libraryMapErrorString);
            }
        }

        #endregion


        #region Common Helper Methods

        /// <summary>
        /// Common helper method to arrive at a stripped title
        /// </summary>
        /// <param name="title">Book title string to strip.</param>
        /// <returns>Title stripped of ":", ",", and ";"</returns>
        private string BuildStrippedTitle(string title)
        {
            return title.Replace(" :", "").Replace(":", "").Replace(",", "").Replace(";", "");
        }

        /// <summary>
        /// Common helper method for rasing the ErrorEncountered event on the UI thread.
        /// </summary>
        /// <param name="errorMessage">String containing user-friendly description of the error encountered.</param>
        /// <param name="message">String containing user-friendly description of the error encountered.</param>
        private void RaiseErrorEncounteredEvent(ErrorEncounteredEvent.ErrorType type, string message)
        {
            Deployment.Current.Dispatcher.BeginInvoke(() =>
            {
                ErrorEncountered(this, new ErrorEncounteredEvent(type, message));
            });
        }

        /// <summary>
        /// Common helper method for parsing WorldCat Library Servlet HTML fragments and adding WorldCat library
        /// objects to the ObservableCollection
        /// </summary>
        /// <param name="result">The results of an asynchronous call to the WorldCat Library Servlet.</param>
        private void ParseAndAddLibraries(IAsyncResult result)
        {
            // Get HTML fragment response and parse out the libraries
            WebResponse response = ((HttpWebRequest)result.AsyncState).EndGetResponse(result);
            StreamReader reader = new StreamReader(response.GetResponseStream());
            string htmlFragment = reader.ReadToEnd();
            int start = htmlFragment.IndexOf(tableRowBeginParseString);
            int end = 0;

            while (start != -1)
            {
                end = htmlFragment.IndexOf(tableRowEndParseString, start);

                // Library data is contained between <tr></tr> tags in the HTML fragment
                string temp = htmlFragment.Substring(start, end - (start + libraryParseString.Length));

                if (temp.Contains(libraryParseString))
                {
                    // If the table row contains a <p class="lib"> tag then we've got a library
                    // user the UI thread to add a new library to the ObservableCollection
                    Deployment.Current.Dispatcher.BeginInvoke(() =>
                    {
                        WorldCatLibraries.Add(new WorldCatLibrary(temp));
                    });
                }

                start = htmlFragment.IndexOf(libraryParseString, end);
            }
        }

        /// <summary>
        /// Common helper method for sorting returned WorldCat libraries based on distance and updating the
        /// ObservableCollection to only contain the closest 6 libraries.
        /// </summary>
        private void SortAndUpdateLibraries()
        {
            Deployment.Current.Dispatcher.BeginInvoke(() =>
            {
                var sortedList = from library in WorldCatLibraries.ToList()
                                 orderby library.DistanceAsInt
                                 select library;

                WorldCatLibraries.Clear();

                for (int i = 0; i < sortedList.Count() && WorldCatLibraries.Count < 6; ++i)
                {
                    // Ensure that no duplicate libraries are added to the list
                    if (!WorldCatLibraries.Any(library => library.MapCode.Equals(sortedList.ElementAt(i).MapCode)))
                    {
                        WorldCatLibraries.Add(sortedList.ElementAt(i));
                    }
                }
            });
        }

        #endregion
    }
}
