﻿//===================================================================================================================
//  File:       SellerPage.xaml.cs
//  Authors:    Dan Becker, Dave Langer 
//  Course:     CSS 590 A - Mobile Computing, Spring 2012
//===================================================================================================================
using System;
using System.Windows;
using System.Windows.Navigation;
using ArchitectsUnited.Phix.Model;
using ArchitectsUnited.Phix.ViewModel;
using Microsoft.Phone.Controls;
using Microsoft.Phone.Tasks;

namespace ArchitectsUnited.Phix.Views
{
    /// <summary>
    /// Implements the Presentation to display details for the user-selected book
    /// </summary>
    public partial class BookPanoramaPage : PhoneApplicationPage
    {
        #region Constants

        // String constants for Query String keys
        private const string isbnKey = "ISBN";
        private const string titleKey = "Title";
        private const string subtitleKey = "Subtitle";

        // User-friendly error messages
        private const string invalidBookErrorString = "Selected book is invalid. Please select another title.";

        #endregion


        #region Properties

        /// <summary>
        /// Current user-selected book for the Pano page
        /// </summary>
        private Book Book { get; set; }

        #endregion


        #region Constructor

        public BookPanoramaPage()
        {
            InitializeComponent();

            // Listen to the LibrariesLoaded event raised by the ViewModel
            App.ViewModel.LibrariesLoaded += new EventHandler(LibrariesLoadedHandler);

            // Listen to the ErrorEncountered event raised by the ViewModel
            App.ViewModel.ErrorEncountered += new EventHandler<ViewModel.ErrorEncounteredEvent>(ErrorEncounteredHandler);

            // Bind the ViewModel's WorldCatLibraries collection to the UI 
            LibraryList.ItemsSource = App.ViewModel.WorldCatLibraries;

            // Bind the ViewModel's SellersList collection to the UI 
            SellersList.ItemsSource = App.ViewModel.WorldCatSellers;
        }

        #endregion


        #region Event Handlers

        /// <summary>
        /// Asynchronously retrieve the list of libraries from WorldCat, show a progress bar to 
        /// the user.
        /// </summary>
        protected override void OnNavigatedTo(System.Windows.Navigation.NavigationEventArgs e)
        {
            // Only run asynchronous operations for new searches
            if (e.NavigationMode == NavigationMode.New)
            {
                ShowProgressBars();

                // get the ISBN that was passed to this page as a navigation arg and create a Book object
                if (NavigationContext.QueryString.ContainsKey(isbnKey) &&
                    NavigationContext.QueryString.ContainsKey(titleKey) &&
                    NavigationContext.QueryString.ContainsKey(subtitleKey))
                {
                    string ISBN = NavigationContext.QueryString[isbnKey];
                    string title = Uri.UnescapeDataString(NavigationContext.QueryString[titleKey]);
                    string subtitle = Uri.UnescapeDataString(NavigationContext.QueryString[subtitleKey]);

                    // get the libraries for the passed ISBN
                    App.ViewModel.GetWorldCatLibrariesForBook(ISBN, title, subtitle);

                    // get a book object for the ISBN we are passed
                    Book = App.ViewModel.GetBookByISBN(ISBN);

                    if (Book != null)
                    {
                        bookPanel.Visibility = Visibility.Visible;

                        // set page Title == book title
                        BookPanoramaControl.Title = Book.Title.ToLower();
                        // TODO -for refinement - if title is long, make the text size smaller

                        // fill out the BookDetails

                        if (Book.Authors != null) { BookDetailAuthor.Text = Book.Authors; }
                        if (Book.Description != null) { BookDetailDescription.Text = Book.Description; }

                        // TODO: get bigger image than the thumbnail - if it is available. In many cases it is not.
                        // TODO: if we get a bigger image, set the width of the CoverImage bigger so we can see the big image in all its glory
                        //       right now we have the coverimage set to 246 px wide - double the standard 128px of the thumbnail.

                        //if (book.CoverImage != null) { BookDetailImage.Source = book.CoverImage; }
                        if (Book.Thumbnail != null) { BookDetailImage.Source = Book.Thumbnail; }

                        if (Book.Title != null) { BookDetailTitle.Text = Book.Title; }
                        if (Book.Subtitle != null) { BookDetailSubtitle.Text = Book.Subtitle; }
                        if ((Book.AverageRating != null) && (Book.RatingsCount != null)) { BookDetailRatings.Text = Book.AverageRating + " stars out of 5 (" + Book.RatingsCount + " total ratings)"; }
                        // TOOD BONUS FEATURE: set a hyperlink here so people can go read the reviews
                    }
                    else
                    {
                        // We don't have a good book here
                        AlertUserToInvalidBook();
                    }
                }
                else
                {
                    // We don't have a good book here
                    AlertUserToInvalidBook();
                }
            }

            base.OnNavigatedTo(e);
        }

        /// <summary>
        /// Event handler for the ViewModel's LibrariesLoaded event. This allows the UI to provide
        /// visual cues to the user regarding the asynchronous processing of getting libraries from
        /// world cat.
        /// </summary>
        private void LibrariesLoadedHandler(object sender, EventArgs e)
        {
            HideProgressBars();

            // Display list to user
            LibraryList.Visibility = Visibility.Visible;
            SellersList.Visibility = Visibility.Visible;
        }

        /// <summary>
        /// Event handler for the ViewModel's ErrorEncountered event. This allows the UI to react to
        /// any errors that might occur during asynchronous processing.
        /// </summary>
        void ErrorEncounteredHandler(object sender, ViewModel.ErrorEncounteredEvent e)
        {
            switch (e.Type)
            {
                case ErrorEncounteredEvent.ErrorType.BingMaps:
                case ErrorEncounteredEvent.ErrorType.WorldCatSearch:
                    // An error was encountered with either the Bing Maps API, WorldCat Basic API or Library/Seller servlets
                    HideProgressBars();

                    txtLibraryErrorMessage.Text = e.Message;
                    txtLibraryErrorMessage.Visibility = Visibility.Visible;
                    txtStoreErrorMessage.Text = e.Message;
                    txtStoreErrorMessage.Visibility = Visibility.Visible;
                    break;

                case ErrorEncounteredEvent.ErrorType.BookNotFound:
                    // WorldCat didn't return a OCLC for the book
                    HideProgressBars();

                    txtLibraryErrorMessage.Text = "No WorldCat libraries found. Please try a different book.";
                    txtLibraryErrorMessage.Visibility = Visibility.Visible;
                    txtStoreErrorMessage.Text = "No WorldCat sellers found. Please try a different book.";
                    txtStoreErrorMessage.Visibility = Visibility.Visible;
                    break;
            }
        }

        /// <summary>
        /// Navigate to map page for selected library
        /// </summary>
        private void LibraryMap_Tap(object sender, System.Windows.Input.GestureEventArgs e)
        {
            // Grab the library the user selected
            WorldCatLibrary library = LibraryList.SelectedItem as WorldCatLibrary;

            // Navigate to the Library Map page using the selected Library
            NavigationService.Navigate(new Uri(string.Format("/LibraryMap/{0}/{1}", library.Name, library.MapCode), UriKind.Relative));

            // Signal that the event was handled this prevents the LibrarList_Tap event handler from being fired
            e.Handled = true;
        }

        /// <summary>
        /// Navigate to library page of the book for the selected library
        /// </summary>
        private void LibraryList_Tap(object sender, System.Windows.Input.GestureEventArgs e)
        {
            // Grab the library the user selected
            WorldCatLibrary mLibrary = LibraryList.SelectedItem as WorldCatLibrary;

            // Experimenting with having the Library Uri open in a new webBrowserTask; 
            // comment out the URI Mapper that opens it in Phix's LibraryPage view

            //// Leverage the URI Mapper defined in the App.xaml file, escape Library URI
            //NavigationService.Navigate(new Uri(string.Format("/Library/{0}", Uri.EscapeDataString(mLibrary.LibraryUri)),
            //                        UriKind.Relative));


            // This means we need to do the URI hacking here instead of letting LibraryPage handle it  
            // - now I really wish I'd been more MVVM and put that code in ViewModel instead of View!


            // the uriKey we get from worldcat for the library page needs significant hacking
            // it starts out as a worldcat Uri with a javascript-encoded version of the 
            // destination library's search URL for the term.
            // And different libraries (with different OCLC systems) have different search URL schemas
            //
            // To make things extra-fun, in more than a few cases, the URL that Worldcat has for the
            // library is itself broken; even if we decode it 'correctly' it may still not 
            // point to a valid page. Phix can't help that, so users may get 404s etc.

            // get the string of the Library's URI so we can hack at it
            string libraryUri = mLibrary.LibraryUri;

            string worldcatUrlMarker = "frame&url=";
            // this is a consistent (I hope!) marker for the end of the worldcat part of the Uri 
            // and the start of the destination Uri - so we can remove everything up to & including
            // the marker
            int i = libraryUri.IndexOf(worldcatUrlMarker);
            if (i > 0)
            {
                libraryUri = libraryUri.Remove(0, i + worldcatUrlMarker.Length);
            }

            // Now we need to unescape the Uri string so we can navigate to it
            libraryUri = Uri.UnescapeDataString(libraryUri);

            // Handle exceptions where we need to clean up WorldCat Uri's so they work
            // there is probably a never-ending supply of these exceptions - as many as there are
            // different versions of library software (aka OCLC software)
  
            // Note: SPL uses bibliocommons.com OCLC

            string SPL_ISBN_StartMarker = "identifier%3Aisbnbegin";
            string SPL_ISBN_EndMarker = "isbnend";
            string Alibris_ISBN_StartMarker = "isbnbegin";
  
            libraryUri = libraryUri.Replace(SPL_ISBN_StartMarker, "");
            libraryUri = libraryUri.Replace(SPL_ISBN_EndMarker, "");
            libraryUri = libraryUri.Replace(Alibris_ISBN_StartMarker, "");

            try
            {
                spawnWebBrowser(new Uri(libraryUri, UriKind.Absolute));
            }
            catch (Exception ex)
            {
                // need to catch this exception - maybe it's really time to move this into ViewModel!
                // That way I could use the private RaiseErrorEncounteredEvent
            }

        }

        /// <summary>
        /// Navigate to seller page for selected book
        /// </summary>
        private void SellersList_Tap(object sender, System.Windows.Input.GestureEventArgs e)
        {
            WorldCatSeller mSeller = SellersList.SelectedItem as WorldCatSeller;

            if (mSeller != null)
            {
                //// Leverage the URI Mapper defined in the App.xaml file, escape Seller URI
                //NavigationService.Navigate(new Uri(string.Format("/Seller/{0}", Uri.EscapeDataString(mSeller.Uri)),
                //                           UriKind.Relative));

                // Experimenting with having the seller Uri open in a new webBrowserTask; 
                // comment out the URI Mapper that opens it in Phix's SellerPage view
                spawnWebBrowser(new Uri (mSeller.Uri, UriKind.Absolute));
            }
        }
        /// <summary>
        /// Navigate to Google Books info page for selected book
        /// </summary>
        private void BookDetail_Tap(object sender, System.Windows.Input.GestureEventArgs e)
        {
            spawnWebBrowser(Book.GoogleBooksUri);
        }

        private void spawnWebBrowser(Uri mUri)
        {
            if (mUri!= null)
            {
                WebBrowserTask webBrowserTask = new WebBrowserTask();
                webBrowserTask.Uri = mUri;
                webBrowserTask.Show();
            }
        }

        #endregion


        #region Common Helper Methods

        /// <summary>
        /// Show the UI cues to the user that asynchronous processing is happening.
        /// </summary>
        private void ShowProgressBars()
        {
            // Fire up progress bars
            txtLibrarySearching.Visibility = Visibility.Visible;
            LibraryProgressBar.Visibility = Visibility.Visible;
            LibraryProgressBar.IsIndeterminate = true;
            txtStoreSearching.Visibility = Visibility.Visible;
            StoreProgressBar.Visibility = Visibility.Visible;
            StoreProgressBar.IsIndeterminate = true;
        }

        /// <summary>
        /// Hide the UI cues to the user that asynchronous processing is happening.
        /// </summary>
        private void HideProgressBars()
        {
            // Set IsIndeterminate to false to minimize CPU overhead
            txtLibrarySearching.Visibility = Visibility.Collapsed;
            LibraryProgressBar.Visibility = Visibility.Collapsed;
            LibraryProgressBar.IsIndeterminate = false;
            txtStoreSearching.Visibility = Visibility.Collapsed;
            StoreProgressBar.Visibility = Visibility.Collapsed;
            StoreProgressBar.IsIndeterminate = false;
        }

        /// <summary>
        /// Update the UI to alert the user that somehow an invalid book was selected
        /// </summary>
        private void AlertUserToInvalidBook()
        {
            HideProgressBars();

            txtBookErrorMessage.Text = invalidBookErrorString;
            txtLibraryErrorMessage.Text = invalidBookErrorString;
            txtStoreErrorMessage.Text = invalidBookErrorString;

            txtBookErrorMessage.Visibility = Visibility.Visible;
            txtLibraryErrorMessage.Visibility = Visibility.Visible;
            txtStoreErrorMessage.Visibility = Visibility.Visible;
        }

        #endregion
    }
}