﻿//===================================================================================================================
//  File:       BookSearchResults.xaml.cs
//  Authors:    Dan Becker, Dave Langer 
//  Course:     CSS 590 A - Mobile Computing, Spring 2012
//===================================================================================================================
using System;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Navigation;
using ArchitectsUnited.Phix.Model;
using ArchitectsUnited.Phix.ViewModel;
using Microsoft.Phone.Controls;

namespace ArchitectsUnited.Phix.Views
{
    /// <summary>
    /// Implements the Presentation to for performing book searches using the Google Books Volume List API
    /// </summary>
    public partial class BookSearchResults : PhoneApplicationPage
    {
        #region Properties

        /// <summary>
        /// Cached the search term (e.g., "design patterns") entered by the user.
        /// </summary>
        private string SearchTerm { get; set; }

        #endregion


        #region Constructor

        public BookSearchResults()
        {
            InitializeComponent();

            // Listen to the ErrorEncountered event raised by the ViewModel
            App.ViewModel.ErrorEncountered += new EventHandler<ErrorEncounteredEvent>(ErrorEncounteredHandler);

            // Listen to the LibrariesLoaded event raised by the ViewModel
            App.ViewModel.SearchCompleted += new EventHandler(SearchCompletedHandler);

            // Bind the ViewModel's WorldCatLibraries collection to the UI 
            SearchList.ItemsSource = App.ViewModel.Books;
        }

        #endregion


        #region Event Handlers

        /// <summary>
        /// Asynchronously retrieve the list of libraries from WorldCat, show a progress bar to 
        /// the user.
        /// </summary>
        protected override void OnNavigatedTo(System.Windows.Navigation.NavigationEventArgs e)
        {
            // Only perform asynchrounous processing for new searches
            if (e.NavigationMode == NavigationMode.New)
            {
                // Fire up progress bar
                txtSearching.Visibility = Visibility.Visible;
                ProgressBar.Visibility = Visibility.Visible;
                ProgressBar.IsIndeterminate = true;

                //string mSearchTerm;

                // get parameter searchterm
                System.Collections.Generic.IDictionary<string, string> parameters = this.NavigationContext.QueryString;
                if (parameters.ContainsKey("q"))
                {
                    // send it to the search
                    SearchTerm = parameters["q"];

                    // set the title bar text for user display
                    ApplicationTitle.Text = "phix is searching for ‘" + SearchTerm + "’";
                }
                else
                {
                    // whoops, didn't have a search term! what to do? shouldn't get here...
                    // TODO: how about some REAL exception handling here, chump?
                    SearchTerm = "Gerald Weinberg";
                }

                App.ViewModel.SearchForBook(SearchTerm);
            }

            base.OnNavigatedTo(e);
        }

        /// <summary>
        /// Event handler for the ViewModel's ErrorEncountered event. This allows the UI to provide
        /// visual cues to the user regarding any errors in asynchronous processing.
        /// </summary>
        void ErrorEncounteredHandler(object sender, ErrorEncounteredEvent e)
        {
            // We only care about Google searches for this page.
            if (e.Type == ErrorEncounteredEvent.ErrorType.GoogleSearch)
            {
                // Display message box to user
                MessageBox.Show(e.Message, "Google Search Error", MessageBoxButton.OK);

                if (NavigationService.CanGoBack)
                {
                    // Remove last entry from stack to provide better UX
                    NavigationService.RemoveBackEntry();
                }

                // Navigate back to the main page
                NavigationService.Navigate(new Uri("/Main", UriKind.Relative));
            }
        }

        /// <summary>
        /// Event handler for the ViewModel's SearchCompleted event. This allows the UI to provide
        /// visual cues to the user regarding the asynchronous processing of searching books using
        /// Google's Book API.
        /// </summary>
        private void SearchCompletedHandler(object sender, EventArgs e)
        {
            // Set the application title
            ApplicationTitle.Text = "phix searched for ‘" + SearchTerm + "’";

            txtSearching.Visibility = Visibility.Collapsed;
            ProgressBar.Visibility = Visibility.Collapsed;
            // Set IsIndeterminate to false to minimize CPU overhead
            ProgressBar.IsIndeterminate = false;

            // Display list to user
            SearchList.Visibility = Visibility.Visible;
        }

        /// <summary>
        /// Event handler for the search list. If the user taps on a book in the list, navigate to 
        /// BookPanoramaPage.xaml to display more details about the book
        /// </summary>
        private void SearchList_Tap(object sender, System.Windows.Input.GestureEventArgs e)
        {
            Book selectedBook = SearchList.SelectedItem as Book;

            if (selectedBook != null)
            {
                NavigationService.Navigate(new Uri(string.Format("/Pano/{0}/{1}/{2}", selectedBook.ISBN, Uri.EscapeDataString(selectedBook.Title), Uri.EscapeDataString(selectedBook.Subtitle)), 
                                                   UriKind.Relative));
            }
        }

        #endregion
    }
}