﻿//===================================================================================================================
//  File:       LibraryMap.xaml.cs
//  Authors:    Dan Becker, Dave Langer 
//  Course:     CSS 590 A - Mobile Computing, Spring 2012
//===================================================================================================================
using System.Device.Location;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Media;
using System.Windows.Navigation;
using System.Windows.Shapes;
using ArchitectsUnited.Phix.ViewModel;
using Microsoft.Phone.Controls;
using Microsoft.Phone.Controls.Maps;
using Microsoft.Phone.Tasks;

namespace ArchitectsUnited.Phix.Views
{
    /// <summary>
    /// Implements the Presentation to display the a user-selected Library on the Bing Maps control
    /// </summary>
    public partial class LibraryMap : PhoneApplicationPage
    {
        #region Constants

        // User-friendly string used in case Bing Maps can't find a library location
        private const string libraryNotLocated = "Sorry, couldn't find a location for {0}.";

        #endregion


        #region Properties

        /// <summary>
        /// Cache the user-selected library name.
        /// </summary>
        private string LibraryName { get; set; }

        /// <summary>
        /// Cache the user-selected library location.
        /// </summary>
        private GeoCoordinate LibraryLocation { get; set; }
        

        #endregion


        #region Constructor

        public LibraryMap()
        {
            InitializeComponent();

            // Listen to the ErrorEncountered event raised by the ViewModel
            App.ViewModel.ErrorEncountered += new System.EventHandler<ErrorEncounteredEvent>(ErrorEncounteredHandler);

            // Listen to the LibrariesLoaded event raised by the ViewModel
            App.ViewModel.LibraryMapped += new System.EventHandler<LibraryMappedEvent>(LibraryMapped);

            // Configure Bing Maps control
            libraryMap.CredentialsProvider = new ApplicationIdCredentialsProvider(App.ViewModel.BingMapsKey);
            libraryMap.Mode = new RoadMode();
        }

        #endregion


        #region EventHandlers

        /// <summary>
        /// Asynchronously retrieve the location data for the library from the Bing Maps Location API. 
        /// </summary>
        protected override void OnNavigatedTo(System.Windows.Navigation.NavigationEventArgs e)
        {
            // Only perform asynchronous operations for new mapping requests
            if (e.NavigationMode == NavigationMode.New)
            {
                // Hide the map
                libraryMap.Visibility = Visibility.Collapsed;

                // Fire up progress bar
                txtGettingMap.Visibility = Visibility.Visible;
                ProgressBar.Visibility = Visibility.Visible;
                ProgressBar.IsIndeterminate = true;

                // Cache the library name
                LibraryName = NavigationContext.QueryString["Library"];

                // Asynchronously retrieve long/lat from WorldCat servlet
                App.ViewModel.MapLibrary(NavigationContext.QueryString["MapCode"]);
            }

            base.OnNavigatedTo(e);
        }

        /// <summary>
        /// Event handler for the ViewModel's ErrorEncountered event. This allows the UI to provide
        /// visual cues to the user regarding errors encountered in asynchronous processing of 
        /// getting library location data from WorldCat.
        /// </summary>
        void ErrorEncounteredHandler(object sender, ErrorEncounteredEvent e)
        {
            HideProgressBar();

            // We only care about library mapping operations for this page.
            if (e.Type == ErrorEncounteredEvent.ErrorType.LibraryMap)
            {
                // Display error message to user
                txtMessage.Visibility = Visibility.Visible;
                txtMessage.Text = e.Message;
            }
        }

        /// <summary>
        /// Event handler for the ViewModel's LibraryMapped event. This allows the UI to provide
        /// visual cues to the user regarding the asynchronous processing of getting library location
        /// data from WorldCat.
        /// </summary>
        private void LibraryMapped(object sender, LibraryMappedEvent e)
        {
            HideProgressBar();

            // Display textblock
            txtMessage.Visibility = Visibility.Visible;

            if (string.IsNullOrEmpty(e.Latitude))
            {
                // WorldCat servlet didnt' return a result
                txtMessage.Text = string.Format(libraryNotLocated, LibraryName);

                // make sure the app bar isn't visible - don't want to let user try to map a library w/o location
                ApplicationBar.IsVisible = false;
            }
            else
            {
                // Set the text to the library name
                txtMessage.Text = LibraryName;

                // Set up the map, including the pushpin
                GeoCoordinate geoCoordinate = new GeoCoordinate(double.Parse(e.Latitude), double.Parse(e.Longitude));
                Pushpin pushpin = new Pushpin();

                pushpin.Location = geoCoordinate;
                pushpin.Content = new Rectangle()
                {
                    Fill = new SolidColorBrush(Colors.Red),
                    Height = 50,
                    Width = 25
                };

                libraryMap.Center = geoCoordinate;
                libraryMap.ZoomLevel = 13;
                libraryMap.Children.Add(pushpin);

                // Show map to the user
                libraryMap.Visibility = Visibility.Visible;

                // unhide the app bar so the user can send to Bing Maps if desired
                ApplicationBar.IsVisible = true;
                LibraryLocation = geoCoordinate;

            }
        }
        /// <summary>
        /// Send the library to Bing Maps for directions
        /// </summary>
        private void ApplicationBarMapItem_Click(object sender, System.EventArgs e)
        {
             BingMapsDirectionsTask bingMapsDirectionTask = new BingMapsDirectionsTask();
            LabeledMapLocation LabeledLML = new LabeledMapLocation(LibraryName, LibraryLocation);
            bingMapsDirectionTask.End = LabeledLML;
            bingMapsDirectionTask.Show();
        }


        #endregion


        #region Common Helper Methods

        /// <summary>
        /// Hide the UI cues to the user that asynchronous processing is happening.
        /// </summary>
        private void HideProgressBar()
        {
            // Set IsIndeterminate to false to minimize CPU overhead
            txtGettingMap.Visibility = Visibility.Collapsed;
            ProgressBar.Visibility = Visibility.Collapsed;
            ProgressBar.IsIndeterminate = false;
        }

        #endregion

    }
}