﻿//===================================================================================================================
//  File:       LibraryPage.xaml.cs
//  Authors:    Dan Becker, Dave Langer 
//  Course:     CSS 590 A - Mobile Computing, Spring 2012
//===================================================================================================================
using System;
using System.Windows;
using System.Windows.Controls;
using Microsoft.Phone.Controls;

namespace ArchitectsUnited.Phix.Views
{
    /// <summary>
    /// Implements the Presentation to display the WorldCat library's page for the user-selected book
    /// </summary>
    public partial class LibraryPage : PhoneApplicationPage
    {
        #region Constants

        // The Query String parameter defined in the URI Mapper in App.xaml
        private const string uriKey = "URI";

        #endregion


        #region Constructor 

        public LibraryPage()
        {
            InitializeComponent();
        }

        #endregion


        #region Event Handlers

        /// <summary>
        /// Have browser control navigate to the library's item page
        /// </summary>
        protected override void OnNavigatedTo(System.Windows.Navigation.NavigationEventArgs e)
        {

            // Hide the browser and message block
            browser.Visibility = Visibility.Collapsed;
            txtInvalidLibraryURI.Visibility = Visibility.Collapsed;

            // Fire up progress bar
            txtLoadingLibraryPage.Visibility = Visibility.Visible;
            ProgressBar.Visibility = Visibility.Visible;
            ProgressBar.IsIndeterminate = true;

            string libraryUri;

            if (NavigationContext.QueryString.ContainsKey(uriKey))
            {

                libraryUri = NavigationContext.QueryString[uriKey];

                // the uriKey we get from worldcat for the library page needs significant hacking
                // it starts out as a worldcat Uri with a javascript-encoded version of the 
                // destination library's search URL for the term.
                // And different libraries (with different OCLC systems) have different search URL schemas
                //
                // To make things extra-fun, in more than a few cases, the URL that Worldcat has for the
                // library is itself broken; even if we decode it 'correctly' it may still not 
                // point to a valid page. Phix can't help that, so users may get 404s etc.

                // hack at the Uri that worldcat provides to avoid the Javascript redirect problem

                string worldcatUrlMarker = "frame&url=";
                // this is a consistent (I hope!) marker for the end of the worldcat part of the Uri 
                // and the start of the destination Uri - so we can remove everything up to & including
                // the marker
                int i = libraryUri.IndexOf(worldcatUrlMarker);
                if (i > 0)
                {
                    libraryUri = libraryUri.Remove(0, i + worldcatUrlMarker.Length);
                }
                
                // Now we need to unescape the Uri string so we can navigate to it
                libraryUri = Uri.UnescapeDataString(libraryUri);

                // Handle Seattle Public Library search URL problems
                string SPL_ISBN_StartMarker = "identifier%3Aisbnbegin";
                string SPL_ISBN_EndMarker = "isbnend";
                // these are SPL-specific; no idea if other library URLs get these, but we definitely need to remove
                // for SPL search to work. Doesn't seem to hurt other libraries...
                // Note: SPL uses bibliocommons.com OCLC
                libraryUri = libraryUri.Replace(SPL_ISBN_StartMarker, "");
                libraryUri = libraryUri.Replace(SPL_ISBN_EndMarker, "");

                // TODO: Update the UI with the seller name
                //UpdateUIWithLibraryName(libraryUri);

                //browser.NavigateToString(whereIwantToGo);
                browser.Navigate(new Uri(libraryUri, UriKind.Absolute));

                // TODO: might need to do URL exception handling - maybe use the 'google for the library name'
                // if navigating to the hacked libraryUri throws an exception?

                // TODO: consider spawning this out to a full web browser task instead of making it a 
                // web browser view inside Phix; that would provide full web browsing chrome and UX
                // which would be very helpful for cases where the user has to do more interaction with 
                // the library website. It would also enable easy emailing of links.
                // Of course the same argument could be applied to the seller page... maybe the way to handle it is to have
                // an app bar with a 'open in external browser' option - could also use this for 'share link'...
            }
            else
            {
                // No seller URI passed, display user-friendly message
                HideProgressBar();

                txtInvalidLibraryURI.Visibility = Visibility.Visible;
            }

            base.OnNavigatedTo(e);
        }



        /// <summary>
        /// Event handler for when navigation of the browser to the seller page is completed. 
        /// </summary>
        private void BrowserNavigated(object sender, System.Windows.Navigation.NavigationEventArgs e)
        {
            HideProgressBar();

            // Show browser
            browser.Visibility = Visibility.Visible;
        }

        #endregion


        #region Common Helper Methods

        /// <summary>
        /// Hide the UI cues to the user that asynchronous processing is happening.
        /// </summary>
        private void HideProgressBar()
        {
            // Turn off progress bars
            txtLoadingLibraryPage.Visibility = Visibility.Collapsed;
            ProgressBar.Visibility = Visibility.Collapsed;
            ProgressBar.IsIndeterminate = false;
        }

        #endregion 
    }
}